#!/usr/bin/env python
# -*- coding: utf-8 -*-

import yaml
import pygraphviz as pgv
from fabric.api import run, put, settings
import warnings

import argparse
import os
import shutil


""" Tools for automatically deploying CASU controllers. """

'''
Note on handling warnings:

There is a known bug in an upstream library, Crypto/blockalg. It is used by
the paramiko library, which is used in fabric.

The warnings are filtered out in this tool, using high specificity; if the
version of paramiko or Crypto change (within aptitude repos for ubuntu 16.04 at
the time of writing) or are used differently, this might not catch the warning.


The strategy here uses a `catch_warnings` context manager [1], and filters for
FutureWarnings only in a specific library on a specific line. See [2] for how
to update.

A more basic filtering can drop ALL FutureWarnings from all packages
    warnings.simplefilter("ignore", FutureWarning)
but instead we here define the module and line to ensure new warnings are not
blindly hidden.

    warnings.filterwarnings(
        action="ignore", category=FutureWarning, module="Crypto", lineno=141)


[1] https://docs.python.org/2/library/warnings.html#available-context-managers
[2] https://docs.python.org/2/library/warnings.html#the-warnings-filter

'''

class Deploy:
    """
    Class for performing deployment tasks.
    """

    def __init__(self, project_file_name):
        """
        Parses the configuration files and initializes internal data structures.
        """
        self.proj_name = os.path.splitext(os.path.basename(project_file_name))[0]
        self.project_root = os.path.dirname(os.path.abspath(project_file_name))
        self.sandbox_dir = self.proj_name + '_sandbox'
        self.fabfile_name = os.path.join(self.sandbox_dir, self.proj_name + '.py')

        self.arena = {}
        self.nbg = None
        self.dep = {}


        self.prepared = False

        with open(project_file_name) as project_file:
            project = yaml.safe_load(project_file)

        # Read the .arena file
        with open(os.path.join(self.project_root, project['arena'])) as arena_file:
            self.arena = yaml.safe_load(arena_file)

        # Read the neighborhood graph, so long as one is defined.
        nbg_fname = project.get('nbg', None)
        if nbg_fname is not None and nbg_fname.lower() not in ['none', 'null']:
            # if one was missed but not explicitly so, should emit warning?
            self.nbg = pgv.AGraph(os.path.join(self.project_root, nbg_fname))


        # Read the deployment file
        with open(os.path.join(self.project_root, project['dep'])) as dep_file:
            self.dep = yaml.safe_load(dep_file)

    def prepare(self, layer_select='all', allow_partial=False):
        """
        Prepare deployment in local folder.
        """

        cwd = os.getcwd()
        print('Changing directory to {0}'.format(self.project_root))
        os.chdir(self.project_root)
        print('Preparing files for deployment!')

        # Remove old fabfile, if it exists
        _msg = "written"
        if os.path.exists(self.fabfile_name):
            _msg = "overwritten!"
        print('The file {0} will be {1}'.format(self.fabfile_name, _msg))
        try:
            os.remove(self.fabfile_name)
        except OSError:
            # The fabfile does not exist,
            # no bigie
            pass

        # Collect fabric tasks
        fabfile_tasks = '''
from fabric.api import cd, run, settings, parallel
import warnings
'''
        fabfile_all = '''
def all():
'''

        # Clean up and create new sandbox folder
        sandbox_path = os.path.join(self.project_root, self.sandbox_dir)
        _msg = "created"
        if os.path.exists(sandbox_path):
            _msg = "overwritten"
        print('The folder {0} will be {1}'.format(sandbox_path, _msg))
        try:
            shutil.rmtree(sandbox_path)
        except OSError:
            # The sandbox directory does not exist,
            # no biggie
            pass

        # Create one folder per arena layer
        # with subfolders for each casu
        os.mkdir(self.sandbox_dir)
        os.chdir(self.sandbox_dir)

        # Select particular layers
        selected_layers = self.dep.keys()
        if layer_select != 'all':
            selected_layers = [layer_select]
            if layer_select not in self.dep.keys():
                raise ValueError (
                    "[F] {} is not a layer in this deployment! aborting.".format(
                        layer_select))


        for layer in selected_layers:
            os.mkdir(layer)
            os.chdir(layer)
            for casu in self.arena[layer]:
                if layer not in self.dep or casu not in self.dep[layer]:
                    # we cannot continue with this casu since incomplete info.
                    if allow_partial is True:
                        print "[W] skipping casu {} {} since no deployment spec".format(
                            layer, casu)
                        continue
                    else:
                        # user is probably not aware of the mismatch (typo in
                        # names between files, perhaps?) Raise error.
                        raise ValueError("[F] incomplete info for casu {} (in layer {}): not specified in .dep file. Did you mean to use --allow-partial option?".format(casu, layer))

                os.mkdir(casu)
                os.chdir(casu)
                # Create the .rtc file
                with open(casu+'.rtc','a') as rtc_file:
                    yaml.dump({'name':casu}, rtc_file, default_flow_style=False)
                    yaml.dump({'pub_addr': self.arena[layer][casu]['pub_addr']},
                               rtc_file, default_flow_style=False)
                    yaml.dump({'sub_addr': self.arena[layer][casu]['sub_addr']},
                              rtc_file, default_flow_style=False)
                    yaml.dump({'msg_addr': 'tcp://*:' + self.arena[layer][casu]['msg_addr'].split(':')[-1]},
                              rtc_file, default_flow_style=False)
                    neighbors = {'neighbors':{}} # Data to be written to the .rtc
                    if self.nbg is not None:
                        sg = self.nbg.get_subgraph(layer)
                        out_neighbors = [] # Data read from the .nbg file
                        name_prefix = ''
                        if sg is None:
                            print('WARNING: No connectivity info for layer {0}'.format(layer))
                        else:
                            if sg.has_node(casu):
                                out_neighbors = sg.out_neighbors(casu)
                            elif sg.has_node(layer + '/' + casu):
                                # The CASU name is prefixed with layer name
                                name_prefix = layer + '/'
                                out_neighbors = sg.out_neighbors(name_prefix + casu)
                            else:
                                print('WARNING: No connectivity info for CASU {0}'.format(casu))
                            # Read all the neighbors and populate the .rtc
                            for nb in out_neighbors:
                                side = str(self.nbg.get_edge(name_prefix+casu,nb).attr['label'])
                                nb_full_name = str(nb).split('/')
                                nb_name = nb_full_name[-1]
                                nb_layer = layer
                                if len(nb_full_name) > 1:
                                    nb_layer = nb_full_name[0]
                                neighbors['neighbors'][side] = {'name': nb_name,
                                                                'address': self.arena[nb_layer][nb_name]['msg_addr']}

                    yaml.dump(neighbors, rtc_file, default_flow_style=False)

                # Copy the controller
                shutil.copy(os.path.join(self.project_root,self.dep[layer][casu]['controller']),'.')
                # Copy additional files
                extra = self.dep[layer][casu].get('extra',None)
                if extra:
                    for item in extra:
                        shutil.copy(os.path.join(self.project_root,item),'.')

                # compile extra args string
                _extra_args = self.dep[layer][casu].get('args', [])
                extra_args = " ".join(_extra_args)

                # Append to fabfile
                fabfile_tasks += '''
@parallel
def {task}():
    with warnings.catch_warnings():
        # ignore the CTR crypto IV warning
        warnings.filterwarnings(action="ignore", category=FutureWarning, module="Crypto", lineno=141)
        with settings(host_string='{host}', user='{username}'):
            with cd('{code_dir}'):
                run('./{command} {rtc} {extra_args}')
                                  '''.format(task=(layer+'_'+casu).replace('-','_'),
                                             host=self.dep[layer][casu]['hostname'],
                                             username=self.dep[layer][casu]['user'],
                                             code_dir=os.path.join(self.dep[layer][casu]['prefix'],layer,casu),
                                             command=os.path.basename(self.dep[layer][casu]['controller']),
                                             rtc=casu+'.rtc',
                                             extra_args=extra_args,
                                             )
                fabfile_all += '    {task}()\n'.format(task=(layer+'_'+casu).replace('-','_'))
                os.chdir('..')
            os.chdir(sandbox_path)

        os.chdir(self.project_root)

        # Finalize the fabric file
        with open(self.fabfile_name,'w') as fabfile:
            fabfile.write(fabfile_tasks)
            fabfile.write(fabfile_all)

        print('Preparation done!')
        print('Returning to original directory {0}'.format(cwd))
        os.chdir(cwd)

        self.prepared = True

    def deploy(self, layer_select='all', allow_partial=False):
        """
        Perform deployment by copying files from the sandbox directory
        to their appropriate destinations.

        arguments:
            `layer_select` : choose a single layer, or all layers to deploy to
            `allow_partial`: enable deployment specifications where the dep file
                           : specifies only a subset of the arena file's casus

        """

        if not self.prepared:
            self.prepare(layer_select=layer_select,allow_partial=allow_partial)

        cwd = os.getcwd()

        os.chdir(os.path.join(self.project_root, self.sandbox_dir))

        # Select particular layers
        selected_layers = self.dep.keys()
        if layer_select != 'all':
            selected_layers = [layer_select]
            if layer_select not in self.dep.keys():
                raise ValueError (
                    "[F] {} is not a layer in this deployment! aborting.".format(
                        layer_select))

        for layer in selected_layers:
            print('Deploying layer {0} ...'.format(layer))
            os.chdir(layer)
            for casu in self.dep[layer]:
                os.chdir(casu)
                # Connect to the target machine and deploy the files.
                with settings(host_string = self.dep[layer][casu]['hostname'],
                              user = self.dep[layer][casu]['user']):
                    destdir = os.path.join(self.dep[layer][casu]['prefix'], layer, casu)
                    run('mkdir -p ' + destdir)
                    run('rm -rf ' + os.path.join(destdir,'*'))
                    put('*', destdir)
                    # Give executable permissions to the controller
                    ctrl_name = os.path.basename(self.dep[layer][casu]['controller'])
                    run('chmod +x ' + os.path.join(destdir, ctrl_name))
                os.chdir('..')
            os.chdir('..')

        os.chdir(cwd)

def main():
    parser = argparse.ArgumentParser(description='Transfer controller code to CASUs (physical or simulated)')
    parser.add_argument('project', help='name of .assisi file specifying the project details.')
    # TODO: This is fully implemented yet!
    parser.add_argument('--layer', help='Name of single layer to deploy', default='all')
    parser.add_argument('--prepare', help='Generate rtc files, but skip transfer to target CASUs', action="store_true")
    parser.add_argument('--allow-partial',
                        help='Allow a partially specified deployment to be generated:'
                        'with a complete arena file, if only a subset of casus are '
                        'declared in the dep file, this will be permitted',
                        action='store_true')
    args = parser.parse_args()

    project = Deploy(args.project)
    with warnings.catch_warnings():
        # ignore the CTR warning in paramiko/fabric
        warnings.filterwarnings(
            action="ignore", category=FutureWarning, module="Crypto", lineno=141)

        if args.prepare is True:
            # ONLY prepare -- don't attempt any transfer
            project.prepare(args.layer, args.allow_partial)
        else:
            # the deployment stage does preparation if not already done
            project.deploy(args.layer, args.allow_partial)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tool for debugging communication issues
between ASSISI system components.
"""

import time
import threading
import zmq

from msg import dev_msgs_pb2

class FakeFw:

    def __init__(self):

        self.ctx = zmq.Context(1)

        # Set up the receiver
        self.sub = self.ctx.socket(zmq.SUB)
        self.sub.bind('tcp://*:5556')
        self.sub.setsockopt(zmq.SUBSCRIBE, '')

        # Start the receiver thread
        self.rcv_thread = threading.Thread(target=self.receive)
        self.rcv_thread.start()
        
        self.pub = self.ctx.socket(zmq.PUB)
        self.pub.bind('tcp://*:5555')
        
    def receive(self):
        while True:
            #[name, dev, cmd, data] = self.sub.recv_multipart()
            #print(name,dev,cmd,data)
            print(self.sub.recv_multipart())

    def run(self):
        # Keep sending dummy data
        while(True):
            pattern = dev_msgs_pb2.VibrationPattern()
            pattern.vibe_periods.extend([1000,100,500])
            pattern.vibe_freqs.extend([440,1,220])
            pattern.vibe_amps.extend([50,0,30])
            self.pub.send_multipart(['casu-016','VibrationPattern','On',pattern.SerializeToString()])
            time.sleep(1)
        
if __name__ == "__main__":

    fw = FakeFw()
    
    try:
        fw.run()
    except KeyboardInterrupt:
        print('Bye-bye')





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dev_msgs.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import base_msgs_pb2 as base__msgs__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='dev_msgs.proto',
  package='AssisiMsg',
  syntax='proto2',
  serialized_pb=_b('\n\x0e\x64\x65v_msgs.proto\x12\tAssisiMsg\x1a\x0f\x62\x61se_msgs.proto\"\xe8\x01\n\nRangeArray\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x32\n\x04type\x18\x02 \x01(\x0e\x32 .AssisiMsg.RangeArray.SensorType:\x02IR\x12\x1d\n\rfield_of_view\x18\x03 \x01(\x02:\x06\x30.5236\x12\x14\n\tmin_range\x18\x04 \x01(\x02:\x01\x30\x12\x16\n\tmax_range\x18\x05 \x01(\x02:\x03inf\x12\r\n\x05range\x18\x06 \x03(\x02\x12\x11\n\traw_value\x18\x07 \x03(\x02\"\x14\n\nSensorType\x12\x06\n\x02IR\x10\x00\"M\n\x0bTemperature\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x0c\n\x04temp\x18\x02 \x02(\x02\x12\r\n\x05slope\x18\x03 \x01(\x02\"C\n\x10TemperatureArray\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x0c\n\x04temp\x18\x02 \x03(\x02\"d\n\x17TemperatureWithGradient\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x0c\n\x04temp\x18\x02 \x02(\x02\x12\x0b\n\x03\x64tx\x18\x03 \x02(\x02\x12\x0b\n\x03\x64ty\x18\x04 \x02(\x02\"W\n\x11VibrationSetpoint\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x0c\n\x04\x66req\x18\x02 \x02(\x01\x12\x11\n\tamplitude\x18\x03 \x01(\x01\"r\n\x10VibrationPattern\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x14\n\x0cvibe_periods\x18\x02 \x03(\r\x12\x12\n\nvibe_freqs\x18\x03 \x03(\x01\x12\x11\n\tvibe_amps\x18\x04 \x03(\r\"o\n\x10VibrationReading\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x0c\n\x04\x66req\x18\x02 \x03(\x02\x12\x11\n\tamplitude\x18\x03 \x03(\x02\x12\x17\n\x0f\x61mplitude_stdev\x18\x04 \x03(\x02\"h\n\x15VibrationReadingArray\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12,\n\x07reading\x18\x02 \x03(\x0b\x32\x1b.AssisiMsg.VibrationReading\"S\n\tDiffDrive\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x10\n\x08vel_left\x18\x02 \x02(\x01\x12\x11\n\tvel_right\x18\x03 \x02(\x01\"\x9a\x01\n\x0bObjectArray\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x1d\n\rfield_of_view\x18\x03 \x01(\x02:\x06\x30.5236\x12\x14\n\tmin_range\x18\x04 \x01(\x02:\x01\x30\x12\x16\n\tmax_range\x18\x05 \x01(\x02:\x03inf\x12\r\n\x05range\x18\x06 \x03(\x02\x12\x0c\n\x04type\x18\x07 \x03(\t\"?\n\x07\x41irflow\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x11\n\tintensity\x18\x02 \x02(\x02\"Y\n\x0e\x41irflowReading\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x11\n\tintensity\x18\x02 \x02(\x02\x12\x11\n\tdirection\x18\x03 \x02(\x02')
  ,
  dependencies=[base__msgs__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_RANGEARRAY_SENSORTYPE = _descriptor.EnumDescriptor(
  name='SensorType',
  full_name='AssisiMsg.RangeArray.SensorType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='IR', index=0, number=0,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=259,
  serialized_end=279,
)
_sym_db.RegisterEnumDescriptor(_RANGEARRAY_SENSORTYPE)


_RANGEARRAY = _descriptor.Descriptor(
  name='RangeArray',
  full_name='AssisiMsg.RangeArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.RangeArray.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='AssisiMsg.RangeArray.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='field_of_view', full_name='AssisiMsg.RangeArray.field_of_view', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.5236),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='min_range', full_name='AssisiMsg.RangeArray.min_range', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_range', full_name='AssisiMsg.RangeArray.max_range', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=1e10000,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='range', full_name='AssisiMsg.RangeArray.range', index=5,
      number=6, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='raw_value', full_name='AssisiMsg.RangeArray.raw_value', index=6,
      number=7, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _RANGEARRAY_SENSORTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=47,
  serialized_end=279,
)


_TEMPERATURE = _descriptor.Descriptor(
  name='Temperature',
  full_name='AssisiMsg.Temperature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.Temperature.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='temp', full_name='AssisiMsg.Temperature.temp', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='slope', full_name='AssisiMsg.Temperature.slope', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=281,
  serialized_end=358,
)


_TEMPERATUREARRAY = _descriptor.Descriptor(
  name='TemperatureArray',
  full_name='AssisiMsg.TemperatureArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.TemperatureArray.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='temp', full_name='AssisiMsg.TemperatureArray.temp', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=360,
  serialized_end=427,
)


_TEMPERATUREWITHGRADIENT = _descriptor.Descriptor(
  name='TemperatureWithGradient',
  full_name='AssisiMsg.TemperatureWithGradient',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.TemperatureWithGradient.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='temp', full_name='AssisiMsg.TemperatureWithGradient.temp', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dtx', full_name='AssisiMsg.TemperatureWithGradient.dtx', index=2,
      number=3, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dty', full_name='AssisiMsg.TemperatureWithGradient.dty', index=3,
      number=4, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=429,
  serialized_end=529,
)


_VIBRATIONSETPOINT = _descriptor.Descriptor(
  name='VibrationSetpoint',
  full_name='AssisiMsg.VibrationSetpoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.VibrationSetpoint.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='freq', full_name='AssisiMsg.VibrationSetpoint.freq', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='amplitude', full_name='AssisiMsg.VibrationSetpoint.amplitude', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=531,
  serialized_end=618,
)


_VIBRATIONPATTERN = _descriptor.Descriptor(
  name='VibrationPattern',
  full_name='AssisiMsg.VibrationPattern',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.VibrationPattern.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='vibe_periods', full_name='AssisiMsg.VibrationPattern.vibe_periods', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='vibe_freqs', full_name='AssisiMsg.VibrationPattern.vibe_freqs', index=2,
      number=3, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='vibe_amps', full_name='AssisiMsg.VibrationPattern.vibe_amps', index=3,
      number=4, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=620,
  serialized_end=734,
)


_VIBRATIONREADING = _descriptor.Descriptor(
  name='VibrationReading',
  full_name='AssisiMsg.VibrationReading',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.VibrationReading.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='freq', full_name='AssisiMsg.VibrationReading.freq', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='amplitude', full_name='AssisiMsg.VibrationReading.amplitude', index=2,
      number=3, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='amplitude_stdev', full_name='AssisiMsg.VibrationReading.amplitude_stdev', index=3,
      number=4, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=736,
  serialized_end=847,
)


_VIBRATIONREADINGARRAY = _descriptor.Descriptor(
  name='VibrationReadingArray',
  full_name='AssisiMsg.VibrationReadingArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.VibrationReadingArray.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='reading', full_name='AssisiMsg.VibrationReadingArray.reading', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=849,
  serialized_end=953,
)


_DIFFDRIVE = _descriptor.Descriptor(
  name='DiffDrive',
  full_name='AssisiMsg.DiffDrive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.DiffDrive.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='vel_left', full_name='AssisiMsg.DiffDrive.vel_left', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='vel_right', full_name='AssisiMsg.DiffDrive.vel_right', index=2,
      number=3, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=955,
  serialized_end=1038,
)


_OBJECTARRAY = _descriptor.Descriptor(
  name='ObjectArray',
  full_name='AssisiMsg.ObjectArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.ObjectArray.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='field_of_view', full_name='AssisiMsg.ObjectArray.field_of_view', index=1,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.5236),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='min_range', full_name='AssisiMsg.ObjectArray.min_range', index=2,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_range', full_name='AssisiMsg.ObjectArray.max_range', index=3,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=1e10000,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='range', full_name='AssisiMsg.ObjectArray.range', index=4,
      number=6, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='AssisiMsg.ObjectArray.type', index=5,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1041,
  serialized_end=1195,
)


_AIRFLOW = _descriptor.Descriptor(
  name='Airflow',
  full_name='AssisiMsg.Airflow',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.Airflow.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='intensity', full_name='AssisiMsg.Airflow.intensity', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1197,
  serialized_end=1260,
)


_AIRFLOWREADING = _descriptor.Descriptor(
  name='AirflowReading',
  full_name='AssisiMsg.AirflowReading',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.AirflowReading.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='intensity', full_name='AssisiMsg.AirflowReading.intensity', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='direction', full_name='AssisiMsg.AirflowReading.direction', index=2,
      number=3, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1262,
  serialized_end=1351,
)

_RANGEARRAY.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
_RANGEARRAY.fields_by_name['type'].enum_type = _RANGEARRAY_SENSORTYPE
_RANGEARRAY_SENSORTYPE.containing_type = _RANGEARRAY
_TEMPERATURE.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
_TEMPERATUREARRAY.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
_TEMPERATUREWITHGRADIENT.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
_VIBRATIONSETPOINT.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
_VIBRATIONPATTERN.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
_VIBRATIONREADING.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
_VIBRATIONREADINGARRAY.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
_VIBRATIONREADINGARRAY.fields_by_name['reading'].message_type = _VIBRATIONREADING
_DIFFDRIVE.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
_OBJECTARRAY.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
_AIRFLOW.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
_AIRFLOWREADING.fields_by_name['header'].message_type = base__msgs__pb2._HEADER
DESCRIPTOR.message_types_by_name['RangeArray'] = _RANGEARRAY
DESCRIPTOR.message_types_by_name['Temperature'] = _TEMPERATURE
DESCRIPTOR.message_types_by_name['TemperatureArray'] = _TEMPERATUREARRAY
DESCRIPTOR.message_types_by_name['TemperatureWithGradient'] = _TEMPERATUREWITHGRADIENT
DESCRIPTOR.message_types_by_name['VibrationSetpoint'] = _VIBRATIONSETPOINT
DESCRIPTOR.message_types_by_name['VibrationPattern'] = _VIBRATIONPATTERN
DESCRIPTOR.message_types_by_name['VibrationReading'] = _VIBRATIONREADING
DESCRIPTOR.message_types_by_name['VibrationReadingArray'] = _VIBRATIONREADINGARRAY
DESCRIPTOR.message_types_by_name['DiffDrive'] = _DIFFDRIVE
DESCRIPTOR.message_types_by_name['ObjectArray'] = _OBJECTARRAY
DESCRIPTOR.message_types_by_name['Airflow'] = _AIRFLOW
DESCRIPTOR.message_types_by_name['AirflowReading'] = _AIRFLOWREADING

RangeArray = _reflection.GeneratedProtocolMessageType('RangeArray', (_message.Message,), dict(
  DESCRIPTOR = _RANGEARRAY,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.RangeArray)
  ))
_sym_db.RegisterMessage(RangeArray)

Temperature = _reflection.GeneratedProtocolMessageType('Temperature', (_message.Message,), dict(
  DESCRIPTOR = _TEMPERATURE,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.Temperature)
  ))
_sym_db.RegisterMessage(Temperature)

TemperatureArray = _reflection.GeneratedProtocolMessageType('TemperatureArray', (_message.Message,), dict(
  DESCRIPTOR = _TEMPERATUREARRAY,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.TemperatureArray)
  ))
_sym_db.RegisterMessage(TemperatureArray)

TemperatureWithGradient = _reflection.GeneratedProtocolMessageType('TemperatureWithGradient', (_message.Message,), dict(
  DESCRIPTOR = _TEMPERATUREWITHGRADIENT,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.TemperatureWithGradient)
  ))
_sym_db.RegisterMessage(TemperatureWithGradient)

VibrationSetpoint = _reflection.GeneratedProtocolMessageType('VibrationSetpoint', (_message.Message,), dict(
  DESCRIPTOR = _VIBRATIONSETPOINT,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.VibrationSetpoint)
  ))
_sym_db.RegisterMessage(VibrationSetpoint)

VibrationPattern = _reflection.GeneratedProtocolMessageType('VibrationPattern', (_message.Message,), dict(
  DESCRIPTOR = _VIBRATIONPATTERN,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.VibrationPattern)
  ))
_sym_db.RegisterMessage(VibrationPattern)

VibrationReading = _reflection.GeneratedProtocolMessageType('VibrationReading', (_message.Message,), dict(
  DESCRIPTOR = _VIBRATIONREADING,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.VibrationReading)
  ))
_sym_db.RegisterMessage(VibrationReading)

VibrationReadingArray = _reflection.GeneratedProtocolMessageType('VibrationReadingArray', (_message.Message,), dict(
  DESCRIPTOR = _VIBRATIONREADINGARRAY,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.VibrationReadingArray)
  ))
_sym_db.RegisterMessage(VibrationReadingArray)

DiffDrive = _reflection.GeneratedProtocolMessageType('DiffDrive', (_message.Message,), dict(
  DESCRIPTOR = _DIFFDRIVE,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.DiffDrive)
  ))
_sym_db.RegisterMessage(DiffDrive)

ObjectArray = _reflection.GeneratedProtocolMessageType('ObjectArray', (_message.Message,), dict(
  DESCRIPTOR = _OBJECTARRAY,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.ObjectArray)
  ))
_sym_db.RegisterMessage(ObjectArray)

Airflow = _reflection.GeneratedProtocolMessageType('Airflow', (_message.Message,), dict(
  DESCRIPTOR = _AIRFLOW,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.Airflow)
  ))
_sym_db.RegisterMessage(Airflow)

AirflowReading = _reflection.GeneratedProtocolMessageType('AirflowReading', (_message.Message,), dict(
  DESCRIPTOR = _AIRFLOWREADING,
  __module__ = 'dev_msgs_pb2'
  # @@protoc_insertion_point(class_scope:AssisiMsg.AirflowReading)
  ))
_sym_db.RegisterMessage(AirflowReading)


# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tools for loading data from CASU logfiles.
"""

import sys
import os
import csv

import scipy.io as sio

def load_from_csv(filepath):
    """
    Load log data from a csv file.
    """
    data = {}

    filename = os.path.basename(filepath)
    casu = ''
    if len(filename) < 25:
        sys.exit('{0} is an invalid CASU log file name!.'.format(filename))
    else:
        casu = filename[20:-4].replace('-','_')
        data[casu] = {}

    with open(filepath) as datafile:
        datareader = csv.reader(datafile,delimiter=';')
        dataid = None
        for row in datareader:
            if len(row) > 2:
                # Guard against incomplete rows
                # (e.g. interrupted program)
                dataid = row[0].replace('-','_')
                t_id = 't_' + dataid
                if not dataid:
                    # Empty data ids appear in some datasets
                    # This actually should not happen
                    # This is a quick fix until we figure out 
                    # the real cause of the problem
                    continue
                if dataid not in data[casu]:
                    # New row id
                    data[casu][dataid] = []
                    data[casu][t_id] = []
                try:
                    ts = float(row[1]) # timestamp
                    data[casu][t_id].append(ts)
                    data[casu][dataid].append([float(x) for x in row[2:]])
                except ValueError:
                    #print('ValueError in row {0}: {1}'.format(datareader.line_num,row))
                    # Accelerometers are currently not providing any data
                    # We don't want to bother users with that
                    pass

        # Check final row (my be incomplete)
        if dataid:
            # Some data were present
            if len(data[casu][dataid]) > 1:
                # At least two rows were present
                if not len(data[casu][dataid][-1]) == len(data[casu][dataid][-2]):
                    # Remove last row if it's incomplete
                    data[casu][dataid] = data[casu][dataid][:-1]

    return data

def process_folder(foldername):
    """
    Recurse into subfolders and process all csv files.
    """
    data = {}

    dirs = os.walk(foldername)
    for (dirpath, dirnames, filenames) in dirs:
        for filename in filenames:
            if filename[-4:] == '.csv':
                new_data = load_from_csv(os.path.join(dirpath,filename))
                if new_data.keys():
                    casu = new_data.keys()[0]
                    if casu in data.keys():
                        print('WARNING: Found more than one logfile for {0}!'.format(casu))
                        print('The output file will contain data from only one logfile!')
                data = dict(data, **new_data)

    return data

def main():
    """
    Main script entry point.
    """
    # TODO: Use argparse and add reasonable options
    # Document usage
    # Consider if the program should take an .assisi file as input?

    data = {}
    
    # The first argument is the name
    # of the file/folder to process
    outname = ''
    if len(sys.argv) < 2:
        sys.exit('Please provide a file or folder to process.')
    elif sys.argv[1][-4:] == '.csv':
        # We are assuming that we need to process a single .csv file
        data = load_from_csv(sys.argv[1])
        outname = sys.argv[1][:-4]
    else:
        # We are assuming that the argument is a folder
        # to be processed. It is assumed that it contains
        # subfolders, each of which corresponds to one CASU.
        # Each subfolder is assumed to contain one or more
        # .csv files.
        data = process_folder(sys.argv[1])
        outname = sys.argv[1].rstrip(os.sep)

    sio.savemat(outname,data,oned_as='column')


if __name__ == '__main__':

    main()
